
package org.llvm.adt.aliases;

import java.util.Arrays;
import org.clank.java.*;
import org.clank.support.*;
import org.clank.support.aliases.*;
import static org.clank.support.Casts.*;
import static org.clank.java.io.*;
import static org.clank.java.std.*;
import static org.llvm.adt.ADTAliases.*;
import static org.llvm.support.llvm.*;
import static org.clank.support.NativePointer.*;
import static org.clank.support.NativeType.*;
import static org.clank.support.Native.*;
import org.llvm.support.*;
import org.llvm.adt.*;
import org.llvm.adt.aliases.*;

/*template <typename ${VALUE_TYPE}> TEMPLATE*/

/// ${FILE_NAME} - Represent a constant reference to an array (0 or more elements
/// consecutively in memory), i.e. a start pointer and a length.  It allows
/// various APIs to take consecutive elements easily and conveniently.
///
/// This class does not own the underlying data, it is expected to be used in
/// situations where the data resides in some other buffer, whose lifetime
/// extends past that of the ${FILE_NAME}. For this reason, it is not in general
/// safe to store an ${FILE_NAME}.
///
/// This is intended to be trivially copyable, so it should be passed by
/// value.
//<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}">
@Converted(kind = Converted.Kind.AUTO, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 30,
 cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}")
//</editor-fold>
public class ${FILE_NAME}${CLASS_SUFFIX}  {
/*public:*/
  /*typedef const ${VALUE_TYPE} *iterator*/
//  public final class iterator extends /*const*/${VALUE_TYPE} /*P*/ { };
  /*typedef const ${VALUE_TYPE} *const_iterator*/
//  public final class const_iterator extends /*const*/${VALUE_TYPE} /*P*/ { };
  /*typedef size_t size_type*/;
  /*typedef std::${REVERSE_ITERATOR}<iterator> ${REVERSE_ITERATOR}*/
//  public final class ${REVERSE_ITERATOR} extends std.${REVERSE_ITERATOR}<iterator>{ };
/*private:*/
  /// The start of the array, in an external buffer.
  private /*const*/${VALUE_POINTER} /*P*/ Data;
  
  /// The number of elements.
  private long/*size_t*/ Length;
/*public:*/
  
  /// Construct an empty ${FILE_NAME}.
  /*implicit*/
  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::${FILE_NAME}${CLASS_SUFFIX}">
  @Converted(kind = Converted.Kind.MANUAL, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 51,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::${FILE_NAME}${CLASS_SUFFIX}")
  //</editor-fold>
  public ${FILE_NAME}() {
    /* : Data(0), Length(0)*/ 
    //START JInit
    this.Data = ${POINTER_FACTORY_METHOD}((${VALUE_TYPE}[]) null);
    this.Length = 0;
    //END JInit
  }

  
  /// Construct an empty ${FILE_NAME} from None.
  /*implicit*/
  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::${FILE_NAME}${CLASS_SUFFIX}">
  @Converted(kind = Converted.Kind.MANUAL, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 54,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::${FILE_NAME}${CLASS_SUFFIX}")
  //</editor-fold>
  public ${FILE_NAME}(NoneType $Prm0) {
    /* : Data(0), Length(0)*/ 
    //START JInit
    this.Data = ${POINTER_FACTORY_METHOD}((${VALUE_TYPE}[]) null);
    this.Length = 0;
    //END JInit
  }

  
  /// Construct an ${FILE_NAME} from a single element.
  /*implicit*/
  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::${FILE_NAME}${CLASS_SUFFIX}">
  @Converted(kind = Converted.Kind.MANUAL, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 57,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::${FILE_NAME}${CLASS_SUFFIX}")
  //</editor-fold>
  public ${FILE_NAME}(/*const*/${VALUE_TYPE} /*&*/ OneElt) {
    /* : Data(&OneElt), Length(1)*/ 
    //START JInit
    this.Data = NativePointer.${POINTER_FACTORY_METHOD}((${VALUE_TYPE}[])${CREATE_ARRAY_FROM_ONE_ELT});
    this.Length = 1;
    //END JInit
  }
  
  // Construct an ${FILE_NAME} using length elements of passed array
  public ${FILE_NAME}(${VALUE_TYPE}[] data, long/*size_t*/ length) {
    this(NativePointer.${POINTER_FACTORY_METHOD}(data), length);
  }
  
  /// Construct an ${FILE_NAME} from a pointer and length.
  /*implicit*/
  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::${FILE_NAME}${CLASS_SUFFIX}">
  @Converted(kind = Converted.Kind.MANUAL, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 61,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::${FILE_NAME}${CLASS_SUFFIX}")
  //</editor-fold>
  public ${FILE_NAME}(/*const*/${VALUE_POINTER} /*P*/ data, long/*size_t*/ length) {
    /* : Data(data), Length(length)*/ 
    //START JInit
    this.Data = data.clone();
    this.Length = length;
    //END JInit
  }

  
  /// Construct an ${FILE_NAME} from a range.
  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::${FILE_NAME}${CLASS_SUFFIX}">
  @Converted(kind = Converted.Kind.MANUAL, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 65,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::${FILE_NAME}${CLASS_SUFFIX}")
  //</editor-fold>
  public ${FILE_NAME}(/*const*/${VALUE_POINTER} /*P*/ begin, /*const*/${VALUE_POINTER} /*P*/ end) {
    /* : Data(begin), Length(end - begin)*/ 
    //START JInit
    this.Data = begin.clone();
    this.Length = end.$sub(begin);
    //END JInit
  }

  
  /// Construct an ${FILE_NAME} from a SmallVector. This is templated in order to
  /// avoid instantiating SmallVectorTemplateCommon${CLASS_SUFFIX} whenever we
  /// copy-construct an ${FILE_NAME}.
  /*template <typename U> TEMPLATE*/
  /*implicit*/
  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::${FILE_NAME}${CLASS_SUFFIX}">
  @Converted(kind = Converted.Kind.MANUAL, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 72,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::${FILE_NAME}${CLASS_SUFFIX}")
  //</editor-fold>
  public </*typename*/ U> ${FILE_NAME}(/*const*/${SMALL_VECTOR_TYPE} /*&*/ Vec) {
    /* : Data(Vec.data()), Length(Vec.size())*/ 
    //START JInit
    this.Data = Vec.data().clone();
    this.Length = Vec.size();
    //END JInit
  }

  
  /// Construct an ${FILE_NAME} from a std::vector.
  /*template <typename A> TEMPLATE*/
  /*implicit*/
  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::${FILE_NAME}${CLASS_SUFFIX}">
  @Converted(kind = Converted.Kind.MANUAL, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 78,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::${FILE_NAME}${CLASS_SUFFIX}")
  //</editor-fold>
  public </*typename*/ A> ${FILE_NAME}(/*const*/${VECTOR_TYPE} /*&*/ Vec) {
    /* : Data(Vec.empty() ? (${VALUE_TYPE} * )0 : &Vec[0]), Length(Vec.size())*/ 
    //START JInit
    this.Data = Vec.empty() ? ${POINTER_FACTORY_METHOD}((${VALUE_TYPE}[]) null) : /*AddrOf*/Vec.ptr$at(0);
    this.Length = Vec.size();
    //END JInit
  }
  public ${FILE_NAME}(/*const*/std.vectorString /*&*/ Vec) {
    /* : Data(Vec.empty() ? (${VALUE_TYPE} * )0 : &Vec[0]), Length(Vec.size())*/ 
    //START JInit
    this.Data = (${VALUE_POINTER}) (Vec.empty() ? ${POINTER_FACTORY_METHOD}((${VALUE_TYPE}[]) null) : /*AddrOf*/Vec.data());
    this.Length = Vec.size();
    //END JInit
  }

  
  /// Construct an ${FILE_NAME} from a C array.
  /*template <size_t N> TEMPLATE*/
  /*implicit*/
  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::${FILE_NAME}${CLASS_SUFFIX}">
  @Converted(kind = Converted.Kind.MANUAL, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 83,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::${FILE_NAME}${CLASS_SUFFIX}")
  //</editor-fold>
  public /*<long*//*size_t*//* N>*/ ${FILE_NAME}(/*const*/${VALUE_TYPE} /*&*/ Arr[]) {
    /* : Data(Arr), Length(N)*/ 
    //START JInit
    this.Data = NativePointer.${POINTER_FACTORY_METHOD}(Arr);
    this.Length = Arr.length;
    //END JInit
  }

  
  /// @}
  /// @name Simple Operations
  /// @{
  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::begin">
  @Converted(kind = Converted.Kind.AUTO, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 97,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::begin")
  //</editor-fold>
  public ${VALUE_POINTER} begin() /*const*/ {
    return Data;
  }

  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::end">
  @Converted(kind = Converted.Kind.AUTO, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 98,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::end")
  //</editor-fold>
  public ${VALUE_POINTER} end() /*const*/ {
    return Data.$add(Length);
  }

  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::rbegin">
  @Converted(kind = Converted.Kind.AUTO, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 100,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::rbegin")
  //</editor-fold>
  public ${REVERSE_ITERATOR} rbegin() /*const*/ {
    return new ${REVERSE_ITERATOR}(end());
  }

  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::rend">
  @Converted(kind = Converted.Kind.AUTO, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 101,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::rend")
  //</editor-fold>
  public ${REVERSE_ITERATOR} rend() /*const*/ {
    return new ${REVERSE_ITERATOR}(begin());
  }

  
  /// empty - Check if the array is empty.
  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::empty">
  @Converted(kind = Converted.Kind.AUTO, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 104,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::empty")
  //</editor-fold>
  public boolean empty() /*const*/ {
    return Length == 0;
  }

  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::data">
  @Converted(kind = Converted.Kind.MANUAL, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 106,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::data")
  //</editor-fold>
  public /*const*/${VALUE_POINTER} /*P*/ data() /*const*/ {
    return Data;
  }

  
  /// size - Get the array size.
  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::size">
  @Converted(kind = Converted.Kind.AUTO, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 109,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::size")
  //</editor-fold>
  public long/*size_t*/ size() /*const*/ {
    return Length;
  }

  
  /// front - Get the first element.
  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::front">
  @Converted(kind = Converted.Kind.MANUAL, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 112,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::front")
  //</editor-fold>
  public /*const*/${VALUE_TYPE} /*&*/ front() /*const*/ {
    return Data.$at(0);
  }

  
  /// back - Get the last element.
  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::back">
  @Converted(kind = Converted.Kind.MANUAL, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 118,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::back")
  //</editor-fold>
  public /*const*/${VALUE_TYPE} /*&*/ back() /*const*/ {
    return Data.$at(Length - 1);
  }

  
  /// equals - Check for element-wise equality.
  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::equals">
  @Converted(kind = Converted.Kind.MANUAL, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 124,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::equals")
  //</editor-fold>
  public boolean equals(${FILE_NAME}${CLASS_SUFFIX} RHS) /*const*/ {
    if ($noteq(Length, RHS.Length)) {
      return false;
    }
    for (long/*size_t*/ i = 0; i != Length; i++)  {
      if ($noteq(Data.$at(i), RHS.Data.$at(i))) {
        return false;
      }
    }
    return true;
  }

  
  /// slice(n) - Chop off the first N elements of the array.
  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::slice">
  @Converted(kind = Converted.Kind.MANUAL, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 134,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::slice")
  //</editor-fold>
  public ${FILE_NAME}${CLASS_SUFFIX} slice(/*uint*/long N) /*const*/ {
    return new ${FILE_NAME}${CLASS_SUFFIX}(data().$add(N), size() - N);
  }

  
  /// slice(n, m) - Chop off the first N elements of the array, and keep M
  /// elements in the array.
  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::slice">
  @Converted(kind = Converted.Kind.MANUAL, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 141,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::slice")
  //</editor-fold>
  public ${FILE_NAME}${CLASS_SUFFIX} slice(/*uint*/long N, /*uint*/long M) /*const*/ {
    return new ${FILE_NAME}${CLASS_SUFFIX}(data().$add(N), M);
  }

  
  /// @}
  /// @name Operator Overloads
  /// @{
  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::operator[]">
  @Converted(kind = Converted.Kind.MANUAL, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 149,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::operator[]")
  //</editor-fold>
  public /*const*/${VALUE_TYPE} /*&*/ $at(long/*size_t*/ Index) /*const*/ {
    return Data.$at(Index);
  }

  
  /// @}
  /// @name Expensive Operations
  /// @{
  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::vec">
  @Converted(kind = Converted.Kind.AUTO, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 157,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::vec")
  //</editor-fold>
  public ${VECTOR_TYPE} vec() /*const*/ {
    return new ${VECTOR_TYPE}(Data, Data.$add(Length), ${DEFAULT_VALUE});
  }

  
  /// @}
  /// @name Conversion operators
  /// @{
  //<editor-fold defaultstate="collapsed" desc="llvm::${FILE_NAME}::operator vector<type-parameter-0-0, allocator<type-parameter-0-0> >">
  @Converted(kind = Converted.Kind.AUTO, source = "${LLVM_SRC}/llvm/include/llvm/ADT/${FILE_NAME}.h", line = 164,
   cmd="jclank.sh ${LLVM_SRC}/llvm/lib/Support/StringRef.cpp -filter=llvm::${FILE_NAME}::operator vector<type-parameter-0-0, allocator<type-parameter-0-0> >")
  //</editor-fold>
  public ${VECTOR_TYPE} $vector/*<type-parameter-0-0, allocator<type-parameter-0-0> >*/() /*const*/ {
    return new ${VECTOR_TYPE}(Data, Data.$add(Length), ${DEFAULT_VALUE});
  }

  @Override
  public String toString() {
    if (this.Length == 0) {
      return "<Empty>";
    }
    StringBuilder out = new StringBuilder("\nArRef{");
    String fmt = "%" + (int)Math.ceil(Math.log10(Length+1)) +"d";
    for (int i = 0; i < Length; i++) {
      out.append("[").append(String.format(fmt, i)).append("]");
      out.append("{").append(this.$at(i)).append("}\n");
    }
    out.append("}ArRef\n");
    return out.toString();
  }

  public void $assign(${FILE_NAME}${CLASS_SUFFIX} other) {
    this.Data = other.Data;
    this.Length = other.Length;
  }
}
